==> 0.0.5.tar.gz: test/field_editor_test.dart <==

library fields.test.field_editor;

import 'dart:mirrors';
import 'package:bench/bench.dart';
import 'package:unittest/unittest.dart';
import '../lib/src/default_transforms.dart';
import '../lib/src/field.dart';
import '../lib/src/field_editor.dart';
import '../lib/src/field_transform.dart';

main() => reflectTests();

// -----------------------------------------------------------------------------
// Setup
// -----------------------------------------------------------------------------

@Setup
void setup() {
  FieldEditor.unregisterAll();
  registerDefaultFieldTransforms();
}

class Rectangle {
  int x, y, w, h;
  Rectangle(this.x, this.y, this.w, this.h);
  String toString() => 'rectangle($x,$y,$w,$h)';
}

class HasFields {
  int i = 7;
  String s = 'snarf';
  Rectangle r = new Rectangle(1, 2, 3, 4);
}

class NotAnEditor { }

class IntEditor extends FieldEditor {
  final Type valueType = int;
  final element = null;
  IntEditor(Field field) : super(field);
}

class StringEditor extends FieldEditor {
  final Type valueType = String; 
  final element = null;
  StringEditor(Field field) : super(field);
}

class RectangleEditorResources extends FieldEditorResources {
  String get style => 'SOME CSS';
  const RectangleEditorResources();
}

class RectangleEditor extends FieldEditor {
  final Type valueType = Rectangle;
  final element = null;
  RectangleEditor(Field field) : super(field, const RectangleEditorResources());
}

// -----------------------------------------------------------------------------
// Register
// -----------------------------------------------------------------------------

@Test('Test that no default field editor is initially registered.')
void testHasNoDefaultFieldEditor() {
  expect(FieldEditor.isDefaultRegistered, isFalse);
}

@Test('Test that register default throws if the given editor type is null.')
void testRegisterDefaultNullThrows() {  
  expect(() {
    FieldEditor.registerDefault(null);
  }, throwsA(isArgumentError));
}

@Test('Test that register throws if the given editor type is null.')
void testRegisterNullEditorTypeThrows() {
  expect(() {
    FieldEditor.register(int, null);
  }, throwsA(isArgumentError));
}

@Test('Test the successful register of a default editor.')
void testRegisterDefault() {
  expect(FieldEditor.isDefaultRegistered, isFalse);
  FieldEditor.registerDefault(IntEditor);
  expect(FieldEditor.isDefaultRegistered, isTrue);
}

@Test('Test that a second attempt to register a default editor throws.')
void testRegisterDefaultTwiceThrows() {
  expect(FieldEditor.isDefaultRegistered, isFalse);
  FieldEditor.registerDefault(IntEditor);  
  expect(() {
    FieldEditor.registerDefault(IntEditor);
  }, throwsA(isStateError));
}

@Test('Test that an attempt to register default a non editor type  throws.')
void testRegisterDefaultNonEditorTypeThrows() {
  expect(FieldEditor.isDefaultRegistered, isFalse);
  expect(() {
    FieldEditor.registerDefault(NotAnEditor);
  }, throwsA(isArgumentError));
}

@Test('Test the successful register of an int editor.')
void testRegisterIntEditor() {
  expect(FieldEditor.isRegistered(int), isFalse);
  FieldEditor.register(int, IntEditor);
  expect(FieldEditor.isRegistered(int), isTrue);
}

@Test('Test that a second attempt to register an int editor throws.')
void testRegisterIntEditorTwiceThrows() {
  expect(FieldEditor.isRegistered(int), isFalse);
  FieldEditor.register(int, IntEditor);  
  expect(() {
    FieldEditor.register(int, IntEditor);
  }, throwsA(isStateError));
}

@Test('Test that an attempt to register a non editor type throws.')
void testRegisterNonEditorTypeThrows() {
  expect(FieldEditor.isRegistered(int), isFalse);
  expect(() {
    FieldEditor.register(int, NotAnEditor);
  }, throwsA(isArgumentError));
}

// -----------------------------------------------------------------------------
// Factory
// -----------------------------------------------------------------------------

@Test('Test that the factory throws if no field editor is registered.')
void testFactoryNoEditorThrows() {
  final obj = new HasFields();
  final mirror = reflect(obj);
  final field = new Field(mirror, const Symbol('i'), int);
  expect(() {
    var editor = new FieldEditor.forField(field);
  }, throwsA(isStateError));
}

@Test('Test that the factory returns the default editor.')
void testFactoryReturnsDefaultEditor() {
  FieldEditor.registerDefault(IntEditor);
  final obj = new HasFields();
  final mirror = reflect(obj);
  final i = new Field(mirror, const Symbol('i'), int);
  final s = new Field(mirror, const Symbol('s'), String);
  final iEditor = new FieldEditor.forField(i);
  final sEditor = new FieldEditor.forField(s);
  expect(iEditor, new isInstanceOf<IntEditor>());
  expect(iEditor.field, equals(i));
  expect(sEditor, new isInstanceOf<IntEditor>());
  expect(sEditor.field, equals(s));
}

@Test('Test that the factory returns registered editors with no default.')
void testFactoryReturnsNoDefaultEditor() {
  FieldEditor.register(int, IntEditor);
  FieldEditor.register(String, StringEditor);
  final obj = new HasFields();
  final mirror = reflect(obj);
  final i = new Field(mirror, const Symbol('i'), int);
  final s = new Field(mirror, const Symbol('s'), String);
  final iEditor = new FieldEditor.forField(i);
  final sEditor = new FieldEditor.forField(s);
  expect(iEditor, new isInstanceOf<IntEditor>());
  expect(iEditor.field, equals(i));
  expect(sEditor, new isInstanceOf<StringEditor>());
  expect(sEditor.field, equals(s));
}

// -----------------------------------------------------------------------------
// Value
// -----------------------------------------------------------------------------

@Test('Test getting the value of an int field in an int editor.')
void testIntFieldIntEditorGetValue() {
  FieldEditor.register(int, IntEditor);
  final obj = new HasFields();
  final mirror = reflect(obj);
  final i = new Field(mirror, const Symbol('i'), int);
  IntEditor editor = new FieldEditor.forField(i);
  expect(editor.isValueReadOnly, isFalse);
  editor.getValue().then(expectAsync1((v) {
    expect(v, equals(7));
  }));
}

@Test('Test getting the value of an int field in a string editor.')
void testIntFieldStringEditorGetValue() {
  FieldEditor.registerDefault(StringEditor);
  final obj = new HasFields();
  final mirror = reflect(obj);
  final i = new Field(mirror, const Symbol('i'), int);
  StringEditor editor = new FieldEditor.forField(i);
  expect(editor.isValueReadOnly, isFalse);
  editor.getValue().then(expectAsync1((v) {
    expect(v, equals('7'));
  }));
}

@Test('Test getting the value of a string field in an int editor throws.')
void testStringFieldIntEditorGetValueThrows() {
  FieldEditor.registerDefault(IntEditor);
  final obj = new HasFields();
  final mirror = reflect(obj);
  final s = new Field(mirror, const Symbol('s'), String);
  IntEditor editor = new FieldEditor.forField(s);
  // This should throw because the value 'snarf' cannot be parsed as an int.
  editor.getValue().catchError(expectAsync1((e) {
    expect(e, new isInstanceOf<FieldTransformError>());
  }));
}

@Test('Test that the value of a rectangle field in a string editor is '
      'read-only when no string -> rectangle transform is registered.')
void testRectagleFieldStringEditorValueReadOnly() {
  FieldEditor.registerDefault(StringEditor);
  final obj = new HasFields();
  final mirror = reflect(obj);
  final r = new Field(mirror, const Symbol('r'), Rectangle);
  StringEditor editor = new FieldEditor.forField(r);
  // There is no registered String -> Rectangle transform so we expect readonly.
  expect(editor.isValueReadOnly, isTrue);
  editor.getValue().then(expectAsync1((v) {
    expect(v, equals('rectangle(1,2,3,4)'));
  }));
}

@Test('Test setting the value of an int field in an int editor.')
void testIntFieldIntEditorSetValue() {
  FieldEditor.register(int, IntEditor);
  final obj = new HasFields();
  final mirror = reflect(obj);
  final i = new Field(mirror, const Symbol('i'), int);
  IntEditor editor = new FieldEditor.forField(i);
  expect(editor.isValueReadOnly, isFalse);
  final verify = expectAsync1((v) {
    expect(v, equals(42));
    expect(obj.i, equals(42));
  });
  editor.setValue(42).then((_) => editor.getValue().then(verify));
}

@Test('Test setting the value of an int field in a string editor.')
void testIntFieldStringEditorSetValue() {
  FieldEditor.registerDefault(StringEditor);
  final obj = new HasFields();
  final mirror = reflect(obj);
  final i = new Field(mirror, const Symbol('i'), int);
  StringEditor editor = new FieldEditor.forField(i);
  expect(editor.isValueReadOnly, isFalse);
  final verify = expectAsync1((v) {
    expect(v, equals('42'));
    expect(obj.i, equals(42));
  });
  editor.setValue('42').then((_) => editor.getValue().then(verify));
}

@Test('Test setting bad value of an int field in a string editor throws.')
void testIntFieldStringEditorSetValueThrows() {
  FieldEditor.registerDefault(StringEditor);
  final obj = new HasFields();
  final mirror = reflect(obj);
  final i = new Field(mirror, const Symbol('i'), int);
  StringEditor editor = new FieldEditor.forField(i);
  expect(editor.isValueReadOnly, isFalse);
  // This should throw because the value cannot be parsed as an int.
  editor.setValue('42bottlesofbeeronthewall').catchError(expectAsync1((e) {
    expect(e, new isInstanceOf<FieldTransformError>());
  }));
}

@Test('Test that setting the value of a rectangle field in a string '
      'editor throws because the editor is read-only.')
void testRectangleFieldStringEditorSetValueThrows() {
  FieldEditor.registerDefault(StringEditor);
  final obj = new HasFields();
  final mirror = reflect(obj);
  final r = new Field(mirror, const Symbol('r'), Rectangle);
  StringEditor editor = new FieldEditor.forField(r);
  // There is no registered String -> Rectangle transform so we expect readonly.
  expect(editor.isValueReadOnly, isTrue);
  // Setting a value when read-only is expected to throw.
  editor.setValue(new Rectangle(5,6,7,8)).catchError(expectAsync1((e) {
    expect(e, new isInstanceOf<UnsupportedError>());
  }));
}

// -----------------------------------------------------------------------------
// Resources
// -----------------------------------------------------------------------------

@Test('Test that a rectangle editor may specify a custom resources type.')
void testRectangleEditorResources() {
  FieldEditor.register(Rectangle, RectangleEditor);
  final obj = new HasFields();
  final mirror = reflect(obj);
  final r = new Field(mirror, const Symbol('r'), Rectangle);
  FieldEditor editor = new FieldEditor.forField(r);
  expect(editor.isValueReadOnly, isFalse);
  expect(editor, const isInstanceOf<RectangleEditor>());
  expect(editor.resources, const isInstanceOf<RectangleEditorResources>());
  expect(editor.resources.style, equals('SOME CSS'));
}

==> 0.0.7.tar.gz: test/field_editor_test.dart <==
@TestGroup('FieldEditor')
library fields.test.field_editor;

import 'dart:mirrors';
import 'package:bench/bench.dart';
import 'package:unittest/unittest.dart';
import '../lib/src/default_transforms.dart';
import '../lib/src/field.dart';
import '../lib/src/field_editor.dart';
import '../lib/src/field_transform.dart';

main() => reflectTests();

// -----------------------------------------------------------------------------
// Setup
// -----------------------------------------------------------------------------

@Setup
void setup() {
  FieldEditor.unregisterAll();
  registerDefaultFieldTransforms();
}

class Rectangle {
  int x, y, w, h;
  Rectangle(this.x, this.y, this.w, this.h);
  String toString() => 'rectangle($x,$y,$w,$h)';
}

class HasFields {
  int i = 7;
  String s = 'snarf';
  Rectangle r = new Rectangle(1, 2, 3, 4);
}

class NotAnEditor { }

class IntEditor extends FieldEditor {
  final Type valueType = int;
  final element = null;
  IntEditor(Field field) : super(field);
}

class StringEditor extends FieldEditor {
  final Type valueType = String; 
  final element = null;
  StringEditor(Field field) : super(field);
}

class RectangleEditorResources extends FieldEditorResources {
  String get style => 'SOME CSS';
  const RectangleEditorResources();
}

class RectangleEditor extends FieldEditor {
  final Type valueType = Rectangle;
  final element = null;
  RectangleEditor(Field field) : super(field, const RectangleEditorResources());
}

// -----------------------------------------------------------------------------
// Register
// -----------------------------------------------------------------------------

@Test('Test that no default field editor is initially registered.')
void testHasNoDefaultFieldEditor() {
  expect(FieldEditor.isDefaultRegistered, isFalse);
}

@Test('Test that register default throws if the given editor type is null.')
@ExpectError(isArgumentError)
void testRegisterDefaultNullThrows() {  
  FieldEditor.registerDefault(null);
}

@Test('Test that register throws if the given editor type is null.')
@ExpectError(isArgumentError)
void testRegisterNullEditorTypeThrows() {
  FieldEditor.register(int, null);
}

@Test('Test the successful register of a default editor.')
void testRegisterDefault() {
  expect(FieldEditor.isDefaultRegistered, isFalse);
  FieldEditor.registerDefault(IntEditor);
  expect(FieldEditor.isDefaultRegistered, isTrue);
}

@Test('Test that a second attempt to register a default editor throws.')
@ExpectError(isStateError)
void testRegisterDefaultTwiceThrows() {
  expect(FieldEditor.isDefaultRegistered, isFalse);
  FieldEditor.registerDefault(IntEditor);  
  FieldEditor.registerDefault(IntEditor);
}

@Test('Test that an attempt to register default a non editor type  throws.')
@ExpectError(isArgumentError)
void testRegisterDefaultNonEditorTypeThrows() {
  expect(FieldEditor.isDefaultRegistered, isFalse);
  FieldEditor.registerDefault(NotAnEditor);
}

@Test('Test the successful register of an int editor.')
void testRegisterIntEditor() {
  expect(FieldEditor.isRegistered(int), isFalse);
  FieldEditor.register(int, IntEditor);
  expect(FieldEditor.isRegistered(int), isTrue);
}

@Test('Test that a second attempt to register an int editor throws.')
@ExpectError(isStateError)
void testRegisterIntEditorTwiceThrows() {
  expect(FieldEditor.isRegistered(int), isFalse);
  FieldEditor.register(int, IntEditor);  
  FieldEditor.register(int, IntEditor);
}

@Test('Test that an attempt to register a non editor type throws.')
@ExpectError(isArgumentError)
void testRegisterNonEditorTypeThrows() {
  expect(FieldEditor.isRegistered(int), isFalse);
  FieldEditor.register(int, NotAnEditor);
}

// -----------------------------------------------------------------------------
// Factory
// -----------------------------------------------------------------------------

@Test('Test that the factory throws if no field editor is registered.')
@ExpectError(isStateError)
void testFactoryNoEditorThrows() {
  final obj = new HasFields();
  final mirror = reflect(obj);
  final field = new Field.forVariable(mirror, const Symbol('i'));
  var editor = new FieldEditor.forField(field);
}

@Test('Test that the factory returns the default editor.')
void testFactoryReturnsDefaultEditor() {
  FieldEditor.registerDefault(IntEditor);
  final obj = new HasFields();
  final mirror = reflect(obj);
  final i = new Field.forVariable(mirror, const Symbol('i'));
  final s = new Field.forVariable(mirror, const Symbol('s'));
  final iEditor = new FieldEditor.forField(i);
  final sEditor = new FieldEditor.forField(s);
  expect(iEditor, new isInstanceOf<IntEditor>());
  expect(iEditor.field, equals(i));
  expect(sEditor, new isInstanceOf<IntEditor>());
  expect(sEditor.field, equals(s));
}

@Test('Test that the factory returns registered editors with no default.')
void testFactoryReturnsNoDefaultEditor() {
  FieldEditor.register(int, IntEditor);
  FieldEditor.register(String, StringEditor);
  final obj = new HasFields();
  final mirror = reflect(obj);
  final i = new Field.forVariable(mirror, const Symbol('i'));
  final s = new Field.forVariable(mirror, const Symbol('s'));
  final iEditor = new FieldEditor.forField(i);
  final sEditor = new FieldEditor.forField(s);
  expect(iEditor, new isInstanceOf<IntEditor>());
  expect(iEditor.field, equals(i));
  expect(sEditor, new isInstanceOf<StringEditor>());
  expect(sEditor.field, equals(s));
}

// -----------------------------------------------------------------------------
// Value
// -----------------------------------------------------------------------------

@Test('Test getting the value of an int field in an int editor.')
void testIntFieldIntEditorGetValue() {
  FieldEditor.register(int, IntEditor);
  final obj = new HasFields();
  final mirror = reflect(obj);
  final i = new Field.forVariable(mirror, const Symbol('i'));
  IntEditor editor = new FieldEditor.forField(i);
  expect(editor.isValueReadOnly, isFalse);
  editor.getValue().then(expectAsync1((v) {
    expect(v, equals(7));
  }));
}

@Test('Test getting the value of an int field in a string editor.')
void testIntFieldStringEditorGetValue() {
  FieldEditor.registerDefault(StringEditor);
  final obj = new HasFields();
  final mirror = reflect(obj);
  final i = new Field.forVariable(mirror, const Symbol('i'));
  StringEditor editor = new FieldEditor.forField(i);
  expect(editor.isValueReadOnly, isFalse);
  editor.getValue().then(expectAsync1((v) {
    expect(v, equals('7'));
  }));
}

@Test('Test getting the value of a string field in an int editor throws.')
void testStringFieldIntEditorGetValueThrows() {
  FieldEditor.registerDefault(IntEditor);
  final obj = new HasFields();
  final mirror = reflect(obj);
  final s = new Field.forVariable(mirror, const Symbol('s'));
  IntEditor editor = new FieldEditor.forField(s);
  // This should throw because the value 'snarf' cannot be parsed as an int.
  editor.getValue().catchError(expectAsync1((e) {
    expect(e, new isInstanceOf<FieldTransformError>());
  }));
}

@Test('Test that the value of a rectangle field in a string editor is '
      'read-only when no string -> rectangle transform is registered.')
void testRectagleFieldStringEditorValueReadOnly() {
  FieldEditor.registerDefault(StringEditor);
  final obj = new HasFields();
  final mirror = reflect(obj);
  final r = new Field.forVariable(mirror, const Symbol('r'));
  StringEditor editor = new FieldEditor.forField(r);
  // There is no registered String -> Rectangle transform so we expect readonly.
  expect(editor.isValueReadOnly, isTrue);
  editor.getValue().then(expectAsync1((v) {
    expect(v, equals('rectangle(1,2,3,4)'));
  }));
}

@Test('Test setting the value of an int field in an int editor.')
void testIntFieldIntEditorSetValue() {
  FieldEditor.register(int, IntEditor);
  final obj = new HasFields();
  final mirror = reflect(obj);
  final i = new Field.forVariable(mirror, const Symbol('i'));
  IntEditor editor = new FieldEditor.forField(i);
  expect(editor.isValueReadOnly, isFalse);
  final verify = expectAsync1((v) {
    expect(v, equals(42));
    expect(obj.i, equals(42));
  });
  editor.setValue(42).then((_) => editor.getValue().then(verify));
}

@Test('Test setting the value of an int field in a string editor.')
void testIntFieldStringEditorSetValue() {
  FieldEditor.registerDefault(StringEditor);
  final obj = new HasFields();
  final mirror = reflect(obj);
  final i = new Field.forVariable(mirror, const Symbol('i'));
  StringEditor editor = new FieldEditor.forField(i);
  expect(editor.isValueReadOnly, isFalse);
  final verify = expectAsync1((v) {
    expect(v, equals('42'));
    expect(obj.i, equals(42));
  });
  editor.setValue('42').then((_) => editor.getValue().then(verify));
}

@Test('Test setting bad value of an int field in a string editor throws.')
void testIntFieldStringEditorSetValueThrows() {
  FieldEditor.registerDefault(StringEditor);
  final obj = new HasFields();
  final mirror = reflect(obj);
  final i = new Field.forVariable(mirror, const Symbol('i'));
  StringEditor editor = new FieldEditor.forField(i);
  expect(editor.isValueReadOnly, isFalse);
  // This should throw because the value cannot be parsed as an int.
  editor.setValue('42bottlesofbeeronthewall').catchError(expectAsync1((e) {
    expect(e, new isInstanceOf<FieldTransformError>());
  }));
}

@Test('Test that setting the value of a rectangle field in a string '
      'editor throws because the editor is read-only.')
void testRectangleFieldStringEditorSetValueThrows() {
  FieldEditor.registerDefault(StringEditor);
  final obj = new HasFields();
  final mirror = reflect(obj);
  final r = new Field.forVariable(mirror, const Symbol('r'));
  StringEditor editor = new FieldEditor.forField(r);
  // There is no registered String -> Rectangle transform so we expect readonly.
  expect(editor.isValueReadOnly, isTrue);
  // Setting a value when read-only is expected to throw.
  editor.setValue(new Rectangle(5,6,7,8)).catchError(expectAsync1((e) {
    expect(e, new isInstanceOf<UnsupportedError>());
  }));
}

// -----------------------------------------------------------------------------
// Resources
// -----------------------------------------------------------------------------

@Test('Test that a rectangle editor may specify a custom resources type.')
void testRectangleEditorResources() {
  FieldEditor.register(Rectangle, RectangleEditor);
  final obj = new HasFields();
  final mirror = reflect(obj);
  final r = new Field.forVariable(mirror, const Symbol('r'));
  FieldEditor editor = new FieldEditor.forField(r);
  expect(editor.isValueReadOnly, isFalse);
  expect(editor, const isInstanceOf<RectangleEditor>());
  expect(editor.resources, const isInstanceOf<RectangleEditorResources>());
  expect(editor.resources.style, equals('SOME CSS'));
}
